"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const concat_1 = require("./iterators/concat");
const distinct_1 = require("./iterators/distinct");
const exclude_1 = require("./iterators/exclude");
const fill_1 = require("./iterators/fill");
const filter_1 = require("./iterators/filter");
const flat_1 = require("./iterators/flat");
const groupJoin_1 = require("./iterators/groupJoin");
const intersect_1 = require("./iterators/intersect");
const join_1 = require("./iterators/join");
const leftJoin_1 = require("./iterators/leftJoin");
const map_1 = require("./iterators/map");
const reverse_1 = require("./iterators/reverse");
const shuffle_1 = require("./iterators/shuffle");
const skip_1 = require("./iterators/skip");
const skipWhile_1 = require("./iterators/skipWhile");
const slice_1 = require("./iterators/slice");
const splice_1 = require("./iterators/splice");
const take_1 = require("./iterators/take");
const takeWhile_1 = require("./iterators/takeWhile");
const average_1 = require("./reducers/average");
const first_1 = require("./reducers/first");
const indexOf_1 = require("./reducers/indexOf");
const last_1 = require("./reducers/last");
const lastIndexOf_1 = require("./reducers/lastIndexOf");
const length_1 = require("./reducers/length");
const max_1 = require("./reducers/max");
const min_1 = require("./reducers/min");
const nth_1 = require("./reducers/nth");
const reduce_1 = require("./reducers/reduce");
const sum_1 = require("./reducers/sum");
const toArray_1 = require("./reducers/toArray");
const toGroups_1 = require("./reducers/toGroups");
const toMap_1 = require("./reducers/toMap");
const toSet_1 = require("./reducers/toSet");
const isIterable_1 = require("./utils/isIterable");
const iterable_1 = require("./utils/iterable");
const iterator_1 = require("./utils/iterator");
function itiriri(source) {
    return new Itiriri(source);
}
exports.default = itiriri;
class Itiriri {
    constructor(source) {
        this.source = source;
    }
    [Symbol.iterator]() {
        return iterator_1.iterator(this.source);
    }
    entries() {
        return new Itiriri(map_1.map(this.source, (elem, idx) => [idx, elem]));
    }
    keys() {
        return new Itiriri(map_1.map(this.source, (_, idx) => idx));
    }
    values() {
        return new Itiriri(this.source);
    }
    forEach(action) {
        for (const [index, value] of this.entries()) {
            action(value, index);
        }
    }
    concat(other) {
        return isIterable_1.isIterable(other) ?
            new Itiriri(concat_1.concat(this.source, other)) :
            new Itiriri(concat_1.concat(this.source, [other]));
    }
    prepend(other) {
        return isIterable_1.isIterable(other) ?
            new Itiriri(concat_1.concat(other, this.source)) :
            new Itiriri(concat_1.concat([other], this.source));
    }
    fill(value, start, end) {
        return new Itiriri(fill_1.fill(this.source, value, start, end));
    }
    nth(index) {
        return nth_1.nth(this.source, index);
    }
    indexOf(element, fromIndex = 0) {
        return indexOf_1.indexOf(this.source, (elem, idx) => idx >= fromIndex && elem === element);
    }
    findIndex(predicate) {
        return indexOf_1.indexOf(this.source, predicate);
    }
    lastIndexOf(element, fromIndex = 0) {
        return lastIndexOf_1.lastIndexOf(this.source, (elem, idx) => idx >= fromIndex && elem === element);
    }
    findLastIndex(predicate) {
        return lastIndexOf_1.lastIndexOf(this.source, predicate);
    }
    length(predicate = alwaysTrue()) {
        return length_1.length(filter_1.filter(this.source, predicate));
    }
    first() {
        return first_1.first(this.source);
    }
    find(predicate) {
        return first_1.first(filter_1.filter(this.source, predicate));
    }
    last() {
        return last_1.last(this.source);
    }
    findLast(predicate) {
        return last_1.last(filter_1.filter(this.source, predicate));
    }
    average(selector = element()) {
        return average_1.average(map_1.map(this.source, selector));
    }
    min(compareFn = comparer()) {
        return min_1.min(this.source, compareFn);
    }
    max(compareFn = comparer()) {
        return max_1.max(this.source, compareFn);
    }
    sum(selector = element()) {
        return sum_1.sum(map_1.map(this.source, selector));
    }
    reduce(callback, initialValue) {
        return reduce_1.reduce(this.source, callback, initialValue);
    }
    reduceRight(callback, initialValue) {
        return reduce_1.reduce(reverse_1.reverse(this.source), callback, initialValue);
    }
    every(predicate) {
        return indexOf_1.indexOf(this.source, (e, i) => !predicate(e, i)) === -1;
    }
    some(predicate) {
        return indexOf_1.indexOf(this.source, predicate) !== -1;
    }
    includes(element, fromIndex = 0) {
        return this.some((elem, idx) => idx >= fromIndex && elem === element);
    }
    sort(compareFn = comparer()) {
        const source = this.source;
        const sortable = iterable_1.iterable(function* () {
            yield* toArray_1.toArray(source).sort(compareFn);
        });
        return new Itiriri(sortable);
    }
    shuffle() {
        return new Itiriri(shuffle_1.shuffle(this.source));
    }
    reverse() {
        return new Itiriri(reverse_1.reverse(this.source));
    }
    filter(predicate) {
        return new Itiriri(filter_1.filter(this.source, predicate));
    }
    take(count) {
        return new Itiriri(take_1.take(this.source, count));
    }
    takeWhile(predicate) {
        return new Itiriri(takeWhile_1.takeWhile(this.source, predicate));
    }
    skip(count) {
        return new Itiriri(skip_1.skip(this.source, count));
    }
    skipWhile(predicate) {
        return new Itiriri(skipWhile_1.skipWhile(this.source, predicate));
    }
    slice(start, end) {
        return new Itiriri(slice_1.slice(this.source, start, end));
    }
    splice(start, deleteCount, ...items) {
        return new Itiriri(splice_1.splice(this.source, start, deleteCount, items));
    }
    map(selector) {
        return new Itiriri(map_1.map(this.source, selector));
    }
    flat(selector) {
        return new Itiriri(flat_1.flat(map_1.map(this.source, selector)));
    }
    groupBy(keySelector, valueSelector = x => x) {
        const source = this.source;
        const groups = iterable_1.iterable(function* () {
            yield* toGroups_1.toGroups(source, keySelector, valueSelector);
        });
        const result = map_1.map(groups, elem => [elem[0], new Itiriri(elem[1])]);
        return new Itiriri(result);
    }
    distinct(selector = element()) {
        return new Itiriri(distinct_1.distinct(this.source, selector));
    }
    exclude(other, selector = element()) {
        return new Itiriri(exclude_1.exclude(this.source, other, selector));
    }
    intersect(other, selector = element()) {
        return new Itiriri(intersect_1.intersect(this.source, other, selector));
    }
    union(other, selector = element()) {
        return new Itiriri(distinct_1.distinct(concat_1.concat(this.source, other), selector));
    }
    join(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = join_1.join(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new Itiriri(iterator);
    }
    leftJoin(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = leftJoin_1.leftJoin(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new Itiriri(iterator);
    }
    rightJoin(other, rightKeySelector, leftKeySelector, joinSelector) {
        const iterator = leftJoin_1.leftJoin(other, this.source, rightKeySelector, leftKeySelector, joinSelector);
        return new Itiriri(iterator);
    }
    groupJoin(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = groupJoin_1.groupJoin(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new Itiriri(iterator);
    }
    toArray(selector) {
        return selector ? toArray_1.toArray(map_1.map(this.source, selector)) :
            toArray_1.toArray(this.source);
    }
    toMap(keySelector = element(), valueSelector = element()) {
        return toMap_1.toMap(this.source, keySelector, valueSelector);
    }
    toGroups(keySelector = element(), valueSelector = element()) {
        return toGroups_1.toGroups(this.source, keySelector, valueSelector);
    }
    toSet(selector = element()) {
        return toSet_1.toSet(map_1.map(this.source, selector));
    }
    toString() {
        return toArray_1.toArray(this.source).toString();
    }
}
function element() {
    return (e) => e;
}
function alwaysTrue() {
    return () => true;
}
function comparer() {
    return (a, b) => a === b ? 0 : (a > b ? 1 : -1);
}

//# sourceMappingURL=index.js.map
